#include "common.h"
#include <string.h>
#include <arpa/inet.h>
#include <sys/select.h>

void make_rrq_frame(const char* filename, char* frame)
{
	*((uint16_t*) frame) = htons(OPCODE_RRQ);
	strncpy(frame + 2, filename, FILENAME_LEN);
	strncpy(frame + 2 + strlen(filename) + 1, "octet", 5);
}

void make_wrq_frame(const char* filename, char* frame)
{
	*((uint16_t*) frame) = htons(OPCODE_WRQ);
	strncpy(frame + 2, filename, FILENAME_LEN);
	strncpy(frame + 2 + strlen(filename) + 1, "octet", 5);
}

void make_data_frame(uint16_t blocknum, char* frame)
{
	*((uint16_t*) frame) = htons(OPCODE_DATA);
	*((uint16_t*) frame + 1) = htons(blocknum);
}

void make_ack_frame(uint16_t blocknum, char* frame)
{
	*((uint16_t*) frame) = htons(OPCODE_ACK);
	*((uint16_t*) frame + 1) = htons(blocknum);
}

void make_error_frame(uint16_t errorcode, const char* errormsg, char* frame)
{
	*((uint16_t*) frame) = htons(OPCODE_ERROR);
	*((uint16_t*) frame + 1) = htons(errorcode);
	strncpy(frame + 4, errormsg, ERRORMSG_LEN);
}

void parse_frame(const char* frame, uint16_t* first, uint16_t* second)
{
	if(first)
		*first = ntohs(*((uint16_t*) frame));
	
	if(second)
		*second = ntohs(*((uint16_t*) frame + 1));
}

int poll_socket(int sck, int timeout)
{
	int sel;
	fd_set rfds;
	struct timeval tv;
	
	FD_ZERO(&rfds);
	FD_SET(sck, &rfds);
	
	tv.tv_sec = timeout;
	tv.tv_usec = 0;
	
	sel = select(sck+1, &rfds, NULL, NULL, &tv);
	return sel;
}

